// Programa simple para comprobar la exactitud de la clasificacion
// a partir de los mapas

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define MAXCLASES 256
#define EXP 100
#define CASO 2     // 0=PAVIAU, 1=PAVIAC, 2=INDIANA, 3=SALINAS

#if CASO==0 // OJO NO ES AQUI
// Imagen de Pavia University
#define FICHERO2 "paviau_gt.pgm"
#define H 340
#define V 610
#define B 103
#elif CASO==1
// Imagen de Pavia Centre
#define FICHERO2 "pavia_centre_gt.pgm"
#define H 715
#define V 1096
#define B 102
#elif CASO==2
// Imagen de Indian pines original de purdue (new)
#define FICHERO2 "indian_pines_gt.pgm"
#define H 145
#define V 145
#define B 220
#elif CASO==3
// Imagen de Salinas corregida (corrected)
#define FICHERO2 "salinas_gt.pgm"
#define H 217
#define V 512
#define B 204
#endif

unsigned char truth[V][H], map[V][H], samples[V][H];
double OA, AA;

void read_gt()
{  FILE *fp; int p, i, j, k, b, maxpc, minpc; size_t a;
   int Nb, Nh, Nv; size_t flag;
   unsigned char c, c1[20], c2[20], c3[20], texto[20], order[20], datatype[20], endian[20];
   fp=fopen(FICHERO2,"r");
   if(fp==NULL) { fprintf(stderr,"Can not open file %s\n",FICHERO2); exit(1); }
   else printf("Open file %s\n",FICHERO2);
   flag=fscanf(fp,"%s\n",texto);
   if(strcasecmp((const char *)texto,"P5")) { fprintf(stderr,"No PGM file\n"); exit(-1); } 
   while((c=fgetc(fp))=='#') { while ((c=fgetc(fp))!='\n') printf("%c",c); printf("\n"); } 
   ungetc(c,fp);
   flag=fscanf(fp,"%s",texto); Nh=atoi((const char*)texto); 
   flag=fscanf(fp,"%s",texto); Nv=atoi((const char*)texto);
   printf(" Size %dx%d\n",Nh,Nv);
   if((Nh!=H)||(Nv!=V)) { perror("no coinciden dim"); exit(-1); }
   flag=fscanf(fp,"%s",texto); c = fgetc(fp); ;
   flag=fread(truth,H*V,1,fp); printf(" Read %lu bytes\n",flag*H*V);
   fclose(fp);
}

void read_map(int num)
{  FILE *fp; int p, i, j, k, b, maxpc, minpc; size_t a;
   int Nb, Nh, Nv; size_t flag; unsigned char fichero[100];
   unsigned char c, c1[20], c2[20], c3[20], texto[20], order[20], datatype[20], endian[20];
   sprintf((unsigned char *)fichero,"map%02d.pgm",num);
   fp=fopen(fichero,"r");
   if(fp==NULL) { fprintf(stderr,"Can not open file %s\n",fichero); exit(1); }
   else printf("Open file %s\n",FICHERO2);
   flag=fscanf(fp,"%s\n",texto);
   if(strcasecmp((const char *)texto,"P5")) { fprintf(stderr,"No PGM file\n"); exit(-1); } 
   else printf("Open file %s\n",fichero);
   while((c=fgetc(fp))=='#') { while ((c=fgetc(fp))!='\n') printf("%c",c); printf("\n"); } 
   ungetc(c,fp);
   flag=fscanf(fp,"%s",texto); Nh=atoi((const char*)texto); 
   flag=fscanf(fp,"%s",texto); Nv=atoi((const char*)texto);
   printf(" Size %dx%d\n",Nh,Nv);
   if((Nh!=H)||(Nv!=V)) { perror("no coinciden dim"); exit(-1); }
   flag=fscanf(fp,"%s",texto); c = fgetc(fp); ;
   flag=fread(map,H*V,1,fp); printf(" Read %lu bytes\n",flag*H*V);
   fclose(fp);
}

void read_txt_samples(int num)
{  FILE *fp; int i,h,v,p, k=0; size_t a, total;
   unsigned char fichero[100];
   sprintf((unsigned char *)fichero,"samples%02d.txt",num);
   fp = fopen(fichero, "r");
   if(fp==NULL) { fprintf(stderr,"Can not open file %s\n",fichero); exit(1); }
   else printf("Open file %s\n",fichero);
   for(v=0;v<V;v++)for(h=0;h<H;h++) samples[v][h]=0;
   if(fp==NULL) { printf("no puedo abrir fichero %s\n",fichero); exit(-1); }
   while(fscanf(fp,"%d %d %d\n",&v,&h,&p)>0) { samples[v][h]=1; k++; }
   printf(" Read %d\n",k);
}

void precision(int num)
{  int i,j, nclases=0;  double oa, aa;
   long int err=0,tot=0, totclase[MAXCLASES], errclase[MAXCLASES];
   for(i=0;i<MAXCLASES;i++) { errclase[i]=0; totclase[i]=0; }
   for(i=0;i<V;i++)for(j=0;j<H;j++) 
   {  if(samples[i][j]==0) totclase[truth[i][j]]++;
      if(truth[i][j]>nclases)nclases=truth[i][j]; }
   for(i=0;i<V;i++)for(j=0;j<H;j++) 
   {  if((samples[i][j]==0)&&(map[i][j]!=0)&&(truth[i][j]!=0)) tot++;
      if((samples[i][j]==0)&&(map[i][j]!=0)&&(truth[i][j]!=0)&&(map[i][j]!=truth[i][j])) 
      {  err++; errclase[truth[i][j]]++; }}
   oa=100-(100.0*err)/tot; OA+=oa;
   aa=0; for(i=1;i<=nclases;i++) aa+=100-(100.0*errclase[i])/totclase[i];
   aa=aa/nclases; AA+=aa;
   printf("nclases=%d\n",nclases);
   printf("tot=%ld, hits=%ld, err=%ld, oa=%f, aa=%f\n",tot,tot-err,err,oa,aa);
   printf("-----------------------------------------------------------\n");
}

main()
{  int i; 
   OA=0; AA=0;
   read_gt();
   for(i=0;i<EXP;i++)
   {  read_map(i);
      read_txt_samples(i);
      precision(i); }
   printf("OA=%3.2f, AA=%3.2f\n",OA/EXP,AA/EXP);
} 
